;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;SRAM Expand v1.0
;made by Deflaktor
;
;bug reports and questions go to deflaktor@oleco.net
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


header
lorom

;this asm hack uses up 146 bytes of data, plus 2 bytes for every ram you want to save to the sram
;in this case it saves 23 ram adresses so it uses up 146+23*2 = 192 bytes

!EmptyAdress = $07C226	;point to empty space in the rom
!SRAMSize = #$02	;SRAM Size amount of bytes = $800*value (game default: #$01)

org $FFD8
	db !SRAMSize	

org $9BCC		;Save Game Routine
	JSL SaveGame
	NOP #2
	;LDX $010A	;replaced opcodes
	;LDA $9CCB,x

org $9E5C		;Load Game Routine (executed when 1 or 2 player is chosen)
	JSL LoadGame
	NOP #2
	;STZ $0DD5	;replaced opcodes
	;STZ $0DB3

org !EmptyAdress	;the SNES adress the code will be inserted to

!CodeSize = CodeEnd-SaveGame
db "STAR"
dw !CodeSize-$01
dw !CodeSize-$01^$FFFF

SaveGame:	PHA				;\
		PHX				;| this makes sure the game goes on without problems
		PHY				;|
		PHP				;/
		SEP #$40			; a set overflow flag means we are saving
		BRA Main

LoadGame: 	STZ $0DD5			;\ replaced opcodes from the LoadGame jump
		STZ $0DB3			;/
		PHA				;\
		PHX				;| this makes sure the game goes on without problems
		PHY				;|
		PHP				;/
		REP #$40			; a cleared overflow flag means we are loading

		;<< SaveGame & LoadGame code start here >>

Main:		REP #$30			; let A,X and Y go 16 bit, since we deal with large numbers > FF
		LDA $010A			; load current save file # (use LDA #$04 to access fourth part of the expanded sram)
		AND #$00FF			; mask high bits to take care of the garbage
		ASL #!SRAMSize+7		; since we have got 3 files, we give every file a fourth of the expanded sram leaving another fourth free
		TAX				; transfer the offset to X
		SEP #$20			; let A go back to 8 bit

		BVC LoadCheck			; if overflow is clear -> Loading

SaveCheck:	LDA #$01			;\ write #$01 into the first byte of the expanded ram since everything stored there is FF
		STA $700800,x			;/ 
		BRA Checked

LoadCheck:	LDA $700800,x			;\ we need to check the first ram of our expanded area since everything stored there is FF
		CMP #$01			;/ the byte must be 01
		BNE Return			; Return if the byte isn't 01

Checked:	INX				; increment x
		LDY #$0000			; store 0 into the index for our loop

Loop:		PHX
		TYX
		LDA (Table),x			; load the ram pointer from the table (low byte)
		STA $00				; store it to a temp ram
		INX
		LDA (Table),x			; load the ram pointer from the table (high byte)
		STA $01				; store it to a temp ram
		TXY
		PLX
		BVC Loading			; if Carry is clear -> Loading

Saving:		LDA ($00)			; load the ram content form the table
		STA $700800,x			; and save it
		BRA GoOn

Loading:	LDA $700800,x			; load the ram from the SRAM
		STA ($00)			; store it to the pointer of the temp ram

GoOn:		INX				; next ram
		INY				; increase index of the loop
		CPX #$DE			; have we reached the end of our table?
		BNE Loop			; loop if we haven't

		;<< SaveGame & LoadGame code end here >>

Return:		BVC GetBack			; if we are loading, we mustn't to execute the replaced opcodes

		PLP
		PLY
		PLX
		PLA

		LDX $010A			;\ replaced opcodes from the SaveGame jump
		LDA $9CCB,x			;/
		RTL

GetBack:	PLP
		PLY
		PLX
		PLA
		RTL

; >>>>>>>>> here you can define which ram adresses should be saved <<<<<<<<<
Table:	dw $C400
CodeEnd: